// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
info: |
    The production CharacterClassEscape :: w evaluates by returning the set of characters containing the sixty-three characters:
    a - z, A - Z, 0 - 9, _
es5id: 15.10.2.12_A3_T1
description: A - Z
---*/

var i0 = "";
for (var j = 0; j < 256; j++)
  i0 += String.fromCharCode(j);
var o0 = "\u0000\u0001\u0002\u0003\u0004\u0005\u0006\u0007\u0008\u0009\u000A\u000B\u000C\u000D\u000E\u000F\u0010\u0011\u0012\u0013\u0014\u0015\u0016\u0017\u0018\u0019\u001A\u001B\u001C\u001D\u001E\u001F\u0020\u0021\u0022\u0023\u0024\u0025\u0026\u0027\u0028\u0029\u002A\u002B\u002C\u002D\u002E\u002F\u003A\u003B\u003C\u003D\u003E\u003F\u0040\u005B\u005C\u005D\u005E\u0060\u007B\u007C\u007D\u007E\u007F\u0080\u0081\u0082\u0083\u0084\u0085\u0086\u0087\u0088\u0089\u008A\u008B\u008C\u008D\u008E\u008F\u0090\u0091\u0092\u0093\u0094\u0095\u0096\u0097\u0098\u0099\u009A\u009B\u009C\u009D\u009E\u009F\u00A0\u00A1\u00A2\u00A3\u00A4\u00A5\u00A6\u00A7\u00A8\u00A9\u00AA\u00AB\u00AC\u00AD\u00AE\u00AF\u00B0\u00B1\u00B2\u00B3\u00B4\u00B5\u00B6\u00B7\u00B8\u00B9\u00BA\u00BB\u00BC\u00BD\u00BE\u00BF\u00C0\u00C1\u00C2\u00C3\u00C4\u00C5\u00C6\u00C7\u00C8\u00C9\u00CA\u00CB\u00CC\u00CD\u00CE\u00CF\u00D0\u00D1\u00D2\u00D3\u00D4\u00D5\u00D6\u00D7\u00D8\u00D9\u00DA\u00DB\u00DC\u00DD\u00DE\u00DF\u00E0\u00E1\u00E2\u00E3\u00E4\u00E5\u00E6\u00E7\u00E8\u00E9\u00EA\u00EB\u00EC\u00ED\u00EE\u00EF\u00F0\u00F1\u00F2\u00F3\u00F4\u00F5\u00F6\u00F7\u00F8\u00F9\u00FA\u00FB\u00FC\u00FD\u00FE\u00FF";
if (i0.replace(/\w+/g, "") !== o0) {
  $ERROR("#0: Error matching character class \w between character 0 and ff");
}

var i1 = "";
for (var j = 256; j < 512; j++)
  i1 += String.fromCharCode(j);
var o1 = i1;
if (i1.replace(/\w+/g, "") !== o1) {
  $ERROR("#1: Error matching character class \w between character 100 and 1ff");
}

var i2 = "";
for (var j = 512; j < 768; j++)
  i2 += String.fromCharCode(j);
var o2 = i2;
if (i2.replace(/\w+/g, "") !== o2) {
  $ERROR("#2: Error matching character class \w between character 200 and 2ff");
}

var i3 = "";
for (var j = 768; j < 1024; j++)
  i3 += String.fromCharCode(j);
var o3 = i3;
if (i3.replace(/\w+/g, "") !== o3) {
  $ERROR("#3: Error matching character class \w between character 300 and 3ff");
}

var i4 = "";
for (var j = 1024; j < 1280; j++)
  i4 += String.fromCharCode(j);
var o4 = i4;
if (i4.replace(/\w+/g, "") !== o4) {
  $ERROR("#4: Error matching character class \w between character 400 and 4ff");
}

var i5 = "";
for (var j = 1280; j < 1536; j++)
  i5 += String.fromCharCode(j);
var o5 = i5;
if (i5.replace(/\w+/g, "") !== o5) {
  $ERROR("#5: Error matching character class \w between character 500 and 5ff");
}

var i6 = "";
for (var j = 1536; j < 1792; j++)
  i6 += String.fromCharCode(j);
var o6 = i6;
if (i6.replace(/\w+/g, "") !== o6) {
  $ERROR("#6: Error matching character class \w between character 600 and 6ff");
}

var i7 = "";
for (var j = 1792; j < 2048; j++)
  i7 += String.fromCharCode(j);
var o7 = i7;
if (i7.replace(/\w+/g, "") !== o7) {
  $ERROR("#7: Error matching character class \w between character 700 and 7ff");
}

var i8 = "";
for (var j = 2048; j < 2304; j++)
  i8 += String.fromCharCode(j);
var o8 = i8;
if (i8.replace(/\w+/g, "") !== o8) {
  $ERROR("#8: Error matching character class \w between character 800 and 8ff");
}

var i9 = "";
for (var j = 2304; j < 2560; j++)
  i9 += String.fromCharCode(j);
var o9 = i9;
if (i9.replace(/\w+/g, "") !== o9) {
  $ERROR("#9: Error matching character class \w between character 900 and 9ff");
}

var i10 = "";
for (var j = 2560; j < 2816; j++)
  i10 += String.fromCharCode(j);
var o10 = i10;
if (i10.replace(/\w+/g, "") !== o10) {
  $ERROR("#10: Error matching character class \w between character a00 and aff");
}

var i11 = "";
for (var j = 2816; j < 3072; j++)
  i11 += String.fromCharCode(j);
var o11 = i11;
if (i11.replace(/\w+/g, "") !== o11) {
  $ERROR("#11: Error matching character class \w between character b00 and bff");
}

var i12 = "";
for (var j = 3072; j < 3328; j++)
  i12 += String.fromCharCode(j);
var o12 = i12;
if (i12.replace(/\w+/g, "") !== o12) {
  $ERROR("#12: Error matching character class \w between character c00 and cff");
}

var i13 = "";
for (var j = 3328; j < 3584; j++)
  i13 += String.fromCharCode(j);
var o13 = i13;
if (i13.replace(/\w+/g, "") !== o13) {
  $ERROR("#13: Error matching character class \w between character d00 and dff");
}

var i14 = "";
for (var j = 3584; j < 3840; j++)
  i14 += String.fromCharCode(j);
var o14 = i14;
if (i14.replace(/\w+/g, "") !== o14) {
  $ERROR("#14: Error matching character class \w between character e00 and eff");
}

var i15 = "";
for (var j = 3840; j < 4096; j++)
  i15 += String.fromCharCode(j);
var o15 = i15;
if (i15.replace(/\w+/g, "") !== o15) {
  $ERROR("#15: Error matching character class \w between character f00 and fff");
}

var i16 = "";
for (var j = 4096; j < 4352; j++)
  i16 += String.fromCharCode(j);
var o16 = i16;
if (i16.replace(/\w+/g, "") !== o16) {
  $ERROR("#16: Error matching character class \w between character 1000 and 10ff");
}

var i17 = "";
for (var j = 4352; j < 4608; j++)
  i17 += String.fromCharCode(j);
var o17 = i17;
if (i17.replace(/\w+/g, "") !== o17) {
  $ERROR("#17: Error matching character class \w between character 1100 and 11ff");
}

var i18 = "";
for (var j = 4608; j < 4864; j++)
  i18 += String.fromCharCode(j);
var o18 = i18;
if (i18.replace(/\w+/g, "") !== o18) {
  $ERROR("#18: Error matching character class \w between character 1200 and 12ff");
}

var i19 = "";
for (var j = 4864; j < 5120; j++)
  i19 += String.fromCharCode(j);
var o19 = i19;
if (i19.replace(/\w+/g, "") !== o19) {
  $ERROR("#19: Error matching character class \w between character 1300 and 13ff");
}

var i20 = "";
for (var j = 5120; j < 5376; j++)
  i20 += String.fromCharCode(j);
var o20 = i20;
if (i20.replace(/\w+/g, "") !== o20) {
  $ERROR("#20: Error matching character class \w between character 1400 and 14ff");
}

var i21 = "";
for (var j = 5376; j < 5632; j++)
  i21 += String.fromCharCode(j);
var o21 = i21;
if (i21.replace(/\w+/g, "") !== o21) {
  $ERROR("#21: Error matching character class \w between character 1500 and 15ff");
}

var i22 = "";
for (var j = 5632; j < 5888; j++)
  i22 += String.fromCharCode(j);
var o22 = i22;
if (i22.replace(/\w+/g, "") !== o22) {
  $ERROR("#22: Error matching character class \w between character 1600 and 16ff");
}

var i23 = "";
for (var j = 5888; j < 6144; j++)
  i23 += String.fromCharCode(j);
var o23 = i23;
if (i23.replace(/\w+/g, "") !== o23) {
  $ERROR("#23: Error matching character class \w between character 1700 and 17ff");
}

var i24 = "";
for (var j = 6144; j < 6400; j++)
  i24 += String.fromCharCode(j);
var o24 = i24;
if (i24.replace(/\w+/g, "") !== o24) {
  $ERROR("#24: Error matching character class \w between character 1800 and 18ff");
}

var i25 = "";
for (var j = 6400; j < 6656; j++)
  i25 += String.fromCharCode(j);
var o25 = i25;
if (i25.replace(/\w+/g, "") !== o25) {
  $ERROR("#25: Error matching character class \w between character 1900 and 19ff");
}

var i26 = "";
for (var j = 6656; j < 6912; j++)
  i26 += String.fromCharCode(j);
var o26 = i26;
if (i26.replace(/\w+/g, "") !== o26) {
  $ERROR("#26: Error matching character class \w between character 1a00 and 1aff");
}

var i27 = "";
for (var j = 6912; j < 7168; j++)
  i27 += String.fromCharCode(j);
var o27 = i27;
if (i27.replace(/\w+/g, "") !== o27) {
  $ERROR("#27: Error matching character class \w between character 1b00 and 1bff");
}

var i28 = "";
for (var j = 7168; j < 7424; j++)
  i28 += String.fromCharCode(j);
var o28 = i28;
if (i28.replace(/\w+/g, "") !== o28) {
  $ERROR("#28: Error matching character class \w between character 1c00 and 1cff");
}

var i29 = "";
for (var j = 7424; j < 7680; j++)
  i29 += String.fromCharCode(j);
var o29 = i29;
if (i29.replace(/\w+/g, "") !== o29) {
  $ERROR("#29: Error matching character class \w between character 1d00 and 1dff");
}

var i30 = "";
for (var j = 7680; j < 7936; j++)
  i30 += String.fromCharCode(j);
var o30 = i30;
if (i30.replace(/\w+/g, "") !== o30) {
  $ERROR("#30: Error matching character class \w between character 1e00 and 1eff");
}

var i31 = "";
for (var j = 7936; j < 8192; j++)
  i31 += String.fromCharCode(j);
var o31 = i31;
if (i31.replace(/\w+/g, "") !== o31) {
  $ERROR("#31: Error matching character class \w between character 1f00 and 1fff");
}

var i32 = "";
for (var j = 8192; j < 8448; j++)
  i32 += String.fromCharCode(j);
var o32 = i32;
if (i32.replace(/\w+/g, "") !== o32) {
  $ERROR("#32: Error matching character class \w between character 2000 and 20ff");
}

var i33 = "";
for (var j = 8448; j < 8704; j++)
  i33 += String.fromCharCode(j);
var o33 = i33;
if (i33.replace(/\w+/g, "") !== o33) {
  $ERROR("#33: Error matching character class \w between character 2100 and 21ff");
}

var i34 = "";
for (var j = 8704; j < 8960; j++)
  i34 += String.fromCharCode(j);
var o34 = i34;
if (i34.replace(/\w+/g, "") !== o34) {
  $ERROR("#34: Error matching character class \w between character 2200 and 22ff");
}

var i35 = "";
for (var j = 8960; j < 9216; j++)
  i35 += String.fromCharCode(j);
var o35 = i35;
if (i35.replace(/\w+/g, "") !== o35) {
  $ERROR("#35: Error matching character class \w between character 2300 and 23ff");
}

var i36 = "";
for (var j = 9216; j < 9472; j++)
  i36 += String.fromCharCode(j);
var o36 = i36;
if (i36.replace(/\w+/g, "") !== o36) {
  $ERROR("#36: Error matching character class \w between character 2400 and 24ff");
}

var i37 = "";
for (var j = 9472; j < 9728; j++)
  i37 += String.fromCharCode(j);
var o37 = i37;
if (i37.replace(/\w+/g, "") !== o37) {
  $ERROR("#37: Error matching character class \w between character 2500 and 25ff");
}

var i38 = "";
for (var j = 9728; j < 9984; j++)
  i38 += String.fromCharCode(j);
var o38 = i38;
if (i38.replace(/\w+/g, "") !== o38) {
  $ERROR("#38: Error matching character class \w between character 2600 and 26ff");
}

var i39 = "";
for (var j = 9984; j < 10240; j++)
  i39 += String.fromCharCode(j);
var o39 = i39;
if (i39.replace(/\w+/g, "") !== o39) {
  $ERROR("#39: Error matching character class \w between character 2700 and 27ff");
}

var i40 = "";
for (var j = 10240; j < 10496; j++)
  i40 += String.fromCharCode(j);
var o40 = i40;
if (i40.replace(/\w+/g, "") !== o40) {
  $ERROR("#40: Error matching character class \w between character 2800 and 28ff");
}

var i41 = "";
for (var j = 10496; j < 10752; j++)
  i41 += String.fromCharCode(j);
var o41 = i41;
if (i41.replace(/\w+/g, "") !== o41) {
  $ERROR("#41: Error matching character class \w between character 2900 and 29ff");
}

var i42 = "";
for (var j = 10752; j < 11008; j++)
  i42 += String.fromCharCode(j);
var o42 = i42;
if (i42.replace(/\w+/g, "") !== o42) {
  $ERROR("#42: Error matching character class \w between character 2a00 and 2aff");
}

var i43 = "";
for (var j = 11008; j < 11264; j++)
  i43 += String.fromCharCode(j);
var o43 = i43;
if (i43.replace(/\w+/g, "") !== o43) {
  $ERROR("#43: Error matching character class \w between character 2b00 and 2bff");
}

var i44 = "";
for (var j = 11264; j < 11520; j++)
  i44 += String.fromCharCode(j);
var o44 = i44;
if (i44.replace(/\w+/g, "") !== o44) {
  $ERROR("#44: Error matching character class \w between character 2c00 and 2cff");
}

var i45 = "";
for (var j = 11520; j < 11776; j++)
  i45 += String.fromCharCode(j);
var o45 = i45;
if (i45.replace(/\w+/g, "") !== o45) {
  $ERROR("#45: Error matching character class \w between character 2d00 and 2dff");
}

var i46 = "";
for (var j = 11776; j < 12032; j++)
  i46 += String.fromCharCode(j);
var o46 = i46;
if (i46.replace(/\w+/g, "") !== o46) {
  $ERROR("#46: Error matching character class \w between character 2e00 and 2eff");
}

var i47 = "";
for (var j = 12032; j < 12288; j++)
  i47 += String.fromCharCode(j);
var o47 = i47;
if (i47.replace(/\w+/g, "") !== o47) {
  $ERROR("#47: Error matching character class \w between character 2f00 and 2fff");
}

var i48 = "";
for (var j = 12288; j < 12544; j++)
  i48 += String.fromCharCode(j);
var o48 = i48;
if (i48.replace(/\w+/g, "") !== o48) {
  $ERROR("#48: Error matching character class \w between character 3000 and 30ff");
}

var i49 = "";
for (var j = 12544; j < 12800; j++)
  i49 += String.fromCharCode(j);
var o49 = i49;
if (i49.replace(/\w+/g, "") !== o49) {
  $ERROR("#49: Error matching character class \w between character 3100 and 31ff");
}

var i50 = "";
for (var j = 12800; j < 13056; j++)
  i50 += String.fromCharCode(j);
var o50 = i50;
if (i50.replace(/\w+/g, "") !== o50) {
  $ERROR("#50: Error matching character class \w between character 3200 and 32ff");
}

var i51 = "";
for (var j = 13056; j < 13312; j++)
  i51 += String.fromCharCode(j);
var o51 = i51;
if (i51.replace(/\w+/g, "") !== o51) {
  $ERROR("#51: Error matching character class \w between character 3300 and 33ff");
}

var i52 = "";
for (var j = 13312; j < 13568; j++)
  i52 += String.fromCharCode(j);
var o52 = i52;
if (i52.replace(/\w+/g, "") !== o52) {
  $ERROR("#52: Error matching character class \w between character 3400 and 34ff");
}

var i53 = "";
for (var j = 13568; j < 13824; j++)
  i53 += String.fromCharCode(j);
var o53 = i53;
if (i53.replace(/\w+/g, "") !== o53) {
  $ERROR("#53: Error matching character class \w between character 3500 and 35ff");
}

var i54 = "";
for (var j = 13824; j < 14080; j++)
  i54 += String.fromCharCode(j);
var o54 = i54;
if (i54.replace(/\w+/g, "") !== o54) {
  $ERROR("#54: Error matching character class \w between character 3600 and 36ff");
}

var i55 = "";
for (var j = 14080; j < 14336; j++)
  i55 += String.fromCharCode(j);
var o55 = i55;
if (i55.replace(/\w+/g, "") !== o55) {
  $ERROR("#55: Error matching character class \w between character 3700 and 37ff");
}

var i56 = "";
for (var j = 14336; j < 14592; j++)
  i56 += String.fromCharCode(j);
var o56 = i56;
if (i56.replace(/\w+/g, "") !== o56) {
  $ERROR("#56: Error matching character class \w between character 3800 and 38ff");
}

var i57 = "";
for (var j = 14592; j < 14848; j++)
  i57 += String.fromCharCode(j);
var o57 = i57;
if (i57.replace(/\w+/g, "") !== o57) {
  $ERROR("#57: Error matching character class \w between character 3900 and 39ff");
}

var i58 = "";
for (var j = 14848; j < 15104; j++)
  i58 += String.fromCharCode(j);
var o58 = i58;
if (i58.replace(/\w+/g, "") !== o58) {
  $ERROR("#58: Error matching character class \w between character 3a00 and 3aff");
}

var i59 = "";
for (var j = 15104; j < 15360; j++)
  i59 += String.fromCharCode(j);
var o59 = i59;
if (i59.replace(/\w+/g, "") !== o59) {
  $ERROR("#59: Error matching character class \w between character 3b00 and 3bff");
}

var i60 = "";
for (var j = 15360; j < 15616; j++)
  i60 += String.fromCharCode(j);
var o60 = i60;
if (i60.replace(/\w+/g, "") !== o60) {
  $ERROR("#60: Error matching character class \w between character 3c00 and 3cff");
}

var i61 = "";
for (var j = 15616; j < 15872; j++)
  i61 += String.fromCharCode(j);
var o61 = i61;
if (i61.replace(/\w+/g, "") !== o61) {
  $ERROR("#61: Error matching character class \w between character 3d00 and 3dff");
}

var i62 = "";
for (var j = 15872; j < 16128; j++)
  i62 += String.fromCharCode(j);
var o62 = i62;
if (i62.replace(/\w+/g, "") !== o62) {
  $ERROR("#62: Error matching character class \w between character 3e00 and 3eff");
}

var i63 = "";
for (var j = 16128; j < 16384; j++)
  i63 += String.fromCharCode(j);
var o63 = i63;
if (i63.replace(/\w+/g, "") !== o63) {
  $ERROR("#63: Error matching character class \w between character 3f00 and 3fff");
}

var i64 = "";
for (var j = 16384; j < 16640; j++)
  i64 += String.fromCharCode(j);
var o64 = i64;
if (i64.replace(/\w+/g, "") !== o64) {
  $ERROR("#64: Error matching character class \w between character 4000 and 40ff");
}

var i65 = "";
for (var j = 16640; j < 16896; j++)
  i65 += String.fromCharCode(j);
var o65 = i65;
if (i65.replace(/\w+/g, "") !== o65) {
  $ERROR("#65: Error matching character class \w between character 4100 and 41ff");
}

var i66 = "";
for (var j = 16896; j < 17152; j++)
  i66 += String.fromCharCode(j);
var o66 = i66;
if (i66.replace(/\w+/g, "") !== o66) {
  $ERROR("#66: Error matching character class \w between character 4200 and 42ff");
}

var i67 = "";
for (var j = 17152; j < 17408; j++)
  i67 += String.fromCharCode(j);
var o67 = i67;
if (i67.replace(/\w+/g, "") !== o67) {
  $ERROR("#67: Error matching character class \w between character 4300 and 43ff");
}

var i68 = "";
for (var j = 17408; j < 17664; j++)
  i68 += String.fromCharCode(j);
var o68 = i68;
if (i68.replace(/\w+/g, "") !== o68) {
  $ERROR("#68: Error matching character class \w between character 4400 and 44ff");
}

var i69 = "";
for (var j = 17664; j < 17920; j++)
  i69 += String.fromCharCode(j);
var o69 = i69;
if (i69.replace(/\w+/g, "") !== o69) {
  $ERROR("#69: Error matching character class \w between character 4500 and 45ff");
}

var i70 = "";
for (var j = 17920; j < 18176; j++)
  i70 += String.fromCharCode(j);
var o70 = i70;
if (i70.replace(/\w+/g, "") !== o70) {
  $ERROR("#70: Error matching character class \w between character 4600 and 46ff");
}

var i71 = "";
for (var j = 18176; j < 18432; j++)
  i71 += String.fromCharCode(j);
var o71 = i71;
if (i71.replace(/\w+/g, "") !== o71) {
  $ERROR("#71: Error matching character class \w between character 4700 and 47ff");
}

var i72 = "";
for (var j = 18432; j < 18688; j++)
  i72 += String.fromCharCode(j);
var o72 = i72;
if (i72.replace(/\w+/g, "") !== o72) {
  $ERROR("#72: Error matching character class \w between character 4800 and 48ff");
}

var i73 = "";
for (var j = 18688; j < 18944; j++)
  i73 += String.fromCharCode(j);
var o73 = i73;
if (i73.replace(/\w+/g, "") !== o73) {
  $ERROR("#73: Error matching character class \w between character 4900 and 49ff");
}

var i74 = "";
for (var j = 18944; j < 19200; j++)
  i74 += String.fromCharCode(j);
var o74 = i74;
if (i74.replace(/\w+/g, "") !== o74) {
  $ERROR("#74: Error matching character class \w between character 4a00 and 4aff");
}

var i75 = "";
for (var j = 19200; j < 19456; j++)
  i75 += String.fromCharCode(j);
var o75 = i75;
if (i75.replace(/\w+/g, "") !== o75) {
  $ERROR("#75: Error matching character class \w between character 4b00 and 4bff");
}

var i76 = "";
for (var j = 19456; j < 19712; j++)
  i76 += String.fromCharCode(j);
var o76 = i76;
if (i76.replace(/\w+/g, "") !== o76) {
  $ERROR("#76: Error matching character class \w between character 4c00 and 4cff");
}

var i77 = "";
for (var j = 19712; j < 19968; j++)
  i77 += String.fromCharCode(j);
var o77 = i77;
if (i77.replace(/\w+/g, "") !== o77) {
  $ERROR("#77: Error matching character class \w between character 4d00 and 4dff");
}

var i78 = "";
for (var j = 19968; j < 20224; j++)
  i78 += String.fromCharCode(j);
var o78 = i78;
if (i78.replace(/\w+/g, "") !== o78) {
  $ERROR("#78: Error matching character class \w between character 4e00 and 4eff");
}

var i79 = "";
for (var j = 20224; j < 20480; j++)
  i79 += String.fromCharCode(j);
var o79 = i79;
if (i79.replace(/\w+/g, "") !== o79) {
  $ERROR("#79: Error matching character class \w between character 4f00 and 4fff");
}

var i80 = "";
for (var j = 20480; j < 20736; j++)
  i80 += String.fromCharCode(j);
var o80 = i80;
if (i80.replace(/\w+/g, "") !== o80) {
  $ERROR("#80: Error matching character class \w between character 5000 and 50ff");
}

var i81 = "";
for (var j = 20736; j < 20992; j++)
  i81 += String.fromCharCode(j);
var o81 = i81;
if (i81.replace(/\w+/g, "") !== o81) {
  $ERROR("#81: Error matching character class \w between character 5100 and 51ff");
}

var i82 = "";
for (var j = 20992; j < 21248; j++)
  i82 += String.fromCharCode(j);
var o82 = i82;
if (i82.replace(/\w+/g, "") !== o82) {
  $ERROR("#82: Error matching character class \w between character 5200 and 52ff");
}

var i83 = "";
for (var j = 21248; j < 21504; j++)
  i83 += String.fromCharCode(j);
var o83 = i83;
if (i83.replace(/\w+/g, "") !== o83) {
  $ERROR("#83: Error matching character class \w between character 5300 and 53ff");
}

var i84 = "";
for (var j = 21504; j < 21760; j++)
  i84 += String.fromCharCode(j);
var o84 = i84;
if (i84.replace(/\w+/g, "") !== o84) {
  $ERROR("#84: Error matching character class \w between character 5400 and 54ff");
}

var i85 = "";
for (var j = 21760; j < 22016; j++)
  i85 += String.fromCharCode(j);
var o85 = i85;
if (i85.replace(/\w+/g, "") !== o85) {
  $ERROR("#85: Error matching character class \w between character 5500 and 55ff");
}

var i86 = "";
for (var j = 22016; j < 22272; j++)
  i86 += String.fromCharCode(j);
var o86 = i86;
if (i86.replace(/\w+/g, "") !== o86) {
  $ERROR("#86: Error matching character class \w between character 5600 and 56ff");
}

var i87 = "";
for (var j = 22272; j < 22528; j++)
  i87 += String.fromCharCode(j);
var o87 = i87;
if (i87.replace(/\w+/g, "") !== o87) {
  $ERROR("#87: Error matching character class \w between character 5700 and 57ff");
}

var i88 = "";
for (var j = 22528; j < 22784; j++)
  i88 += String.fromCharCode(j);
var o88 = i88;
if (i88.replace(/\w+/g, "") !== o88) {
  $ERROR("#88: Error matching character class \w between character 5800 and 58ff");
}

var i89 = "";
for (var j = 22784; j < 23040; j++)
  i89 += String.fromCharCode(j);
var o89 = i89;
if (i89.replace(/\w+/g, "") !== o89) {
  $ERROR("#89: Error matching character class \w between character 5900 and 59ff");
}

var i90 = "";
for (var j = 23040; j < 23296; j++)
  i90 += String.fromCharCode(j);
var o90 = i90;
if (i90.replace(/\w+/g, "") !== o90) {
  $ERROR("#90: Error matching character class \w between character 5a00 and 5aff");
}

var i91 = "";
for (var j = 23296; j < 23552; j++)
  i91 += String.fromCharCode(j);
var o91 = i91;
if (i91.replace(/\w+/g, "") !== o91) {
  $ERROR("#91: Error matching character class \w between character 5b00 and 5bff");
}

var i92 = "";
for (var j = 23552; j < 23808; j++)
  i92 += String.fromCharCode(j);
var o92 = i92;
if (i92.replace(/\w+/g, "") !== o92) {
  $ERROR("#92: Error matching character class \w between character 5c00 and 5cff");
}

var i93 = "";
for (var j = 23808; j < 24064; j++)
  i93 += String.fromCharCode(j);
var o93 = i93;
if (i93.replace(/\w+/g, "") !== o93) {
  $ERROR("#93: Error matching character class \w between character 5d00 and 5dff");
}

var i94 = "";
for (var j = 24064; j < 24320; j++)
  i94 += String.fromCharCode(j);
var o94 = i94;
if (i94.replace(/\w+/g, "") !== o94) {
  $ERROR("#94: Error matching character class \w between character 5e00 and 5eff");
}

var i95 = "";
for (var j = 24320; j < 24576; j++)
  i95 += String.fromCharCode(j);
var o95 = i95;
if (i95.replace(/\w+/g, "") !== o95) {
  $ERROR("#95: Error matching character class \w between character 5f00 and 5fff");
}

var i96 = "";
for (var j = 24576; j < 24832; j++)
  i96 += String.fromCharCode(j);
var o96 = i96;
if (i96.replace(/\w+/g, "") !== o96) {
  $ERROR("#96: Error matching character class \w between character 6000 and 60ff");
}

var i97 = "";
for (var j = 24832; j < 25088; j++)
  i97 += String.fromCharCode(j);
var o97 = i97;
if (i97.replace(/\w+/g, "") !== o97) {
  $ERROR("#97: Error matching character class \w between character 6100 and 61ff");
}

var i98 = "";
for (var j = 25088; j < 25344; j++)
  i98 += String.fromCharCode(j);
var o98 = i98;
if (i98.replace(/\w+/g, "") !== o98) {
  $ERROR("#98: Error matching character class \w between character 6200 and 62ff");
}

var i99 = "";
for (var j = 25344; j < 25600; j++)
  i99 += String.fromCharCode(j);
var o99 = i99;
if (i99.replace(/\w+/g, "") !== o99) {
  $ERROR("#99: Error matching character class \w between character 6300 and 63ff");
}

var i100 = "";
for (var j = 25600; j < 25856; j++)
  i100 += String.fromCharCode(j);
var o100 = i100;
if (i100.replace(/\w+/g, "") !== o100) {
  $ERROR("#100: Error matching character class \w between character 6400 and 64ff");
}

var i101 = "";
for (var j = 25856; j < 26112; j++)
  i101 += String.fromCharCode(j);
var o101 = i101;
if (i101.replace(/\w+/g, "") !== o101) {
  $ERROR("#101: Error matching character class \w between character 6500 and 65ff");
}

var i102 = "";
for (var j = 26112; j < 26368; j++)
  i102 += String.fromCharCode(j);
var o102 = i102;
if (i102.replace(/\w+/g, "") !== o102) {
  $ERROR("#102: Error matching character class \w between character 6600 and 66ff");
}

var i103 = "";
for (var j = 26368; j < 26624; j++)
  i103 += String.fromCharCode(j);
var o103 = i103;
if (i103.replace(/\w+/g, "") !== o103) {
  $ERROR("#103: Error matching character class \w between character 6700 and 67ff");
}

var i104 = "";
for (var j = 26624; j < 26880; j++)
  i104 += String.fromCharCode(j);
var o104 = i104;
if (i104.replace(/\w+/g, "") !== o104) {
  $ERROR("#104: Error matching character class \w between character 6800 and 68ff");
}

var i105 = "";
for (var j = 26880; j < 27136; j++)
  i105 += String.fromCharCode(j);
var o105 = i105;
if (i105.replace(/\w+/g, "") !== o105) {
  $ERROR("#105: Error matching character class \w between character 6900 and 69ff");
}

var i106 = "";
for (var j = 27136; j < 27392; j++)
  i106 += String.fromCharCode(j);
var o106 = i106;
if (i106.replace(/\w+/g, "") !== o106) {
  $ERROR("#106: Error matching character class \w between character 6a00 and 6aff");
}

var i107 = "";
for (var j = 27392; j < 27648; j++)
  i107 += String.fromCharCode(j);
var o107 = i107;
if (i107.replace(/\w+/g, "") !== o107) {
  $ERROR("#107: Error matching character class \w between character 6b00 and 6bff");
}

var i108 = "";
for (var j = 27648; j < 27904; j++)
  i108 += String.fromCharCode(j);
var o108 = i108;
if (i108.replace(/\w+/g, "") !== o108) {
  $ERROR("#108: Error matching character class \w between character 6c00 and 6cff");
}

var i109 = "";
for (var j = 27904; j < 28160; j++)
  i109 += String.fromCharCode(j);
var o109 = i109;
if (i109.replace(/\w+/g, "") !== o109) {
  $ERROR("#109: Error matching character class \w between character 6d00 and 6dff");
}

var i110 = "";
for (var j = 28160; j < 28416; j++)
  i110 += String.fromCharCode(j);
var o110 = i110;
if (i110.replace(/\w+/g, "") !== o110) {
  $ERROR("#110: Error matching character class \w between character 6e00 and 6eff");
}

var i111 = "";
for (var j = 28416; j < 28672; j++)
  i111 += String.fromCharCode(j);
var o111 = i111;
if (i111.replace(/\w+/g, "") !== o111) {
  $ERROR("#111: Error matching character class \w between character 6f00 and 6fff");
}

var i112 = "";
for (var j = 28672; j < 28928; j++)
  i112 += String.fromCharCode(j);
var o112 = i112;
if (i112.replace(/\w+/g, "") !== o112) {
  $ERROR("#112: Error matching character class \w between character 7000 and 70ff");
}

var i113 = "";
for (var j = 28928; j < 29184; j++)
  i113 += String.fromCharCode(j);
var o113 = i113;
if (i113.replace(/\w+/g, "") !== o113) {
  $ERROR("#113: Error matching character class \w between character 7100 and 71ff");
}

var i114 = "";
for (var j = 29184; j < 29440; j++)
  i114 += String.fromCharCode(j);
var o114 = i114;
if (i114.replace(/\w+/g, "") !== o114) {
  $ERROR("#114: Error matching character class \w between character 7200 and 72ff");
}

var i115 = "";
for (var j = 29440; j < 29696; j++)
  i115 += String.fromCharCode(j);
var o115 = i115;
if (i115.replace(/\w+/g, "") !== o115) {
  $ERROR("#115: Error matching character class \w between character 7300 and 73ff");
}

var i116 = "";
for (var j = 29696; j < 29952; j++)
  i116 += String.fromCharCode(j);
var o116 = i116;
if (i116.replace(/\w+/g, "") !== o116) {
  $ERROR("#116: Error matching character class \w between character 7400 and 74ff");
}

var i117 = "";
for (var j = 29952; j < 30208; j++)
  i117 += String.fromCharCode(j);
var o117 = i117;
if (i117.replace(/\w+/g, "") !== o117) {
  $ERROR("#117: Error matching character class \w between character 7500 and 75ff");
}

var i118 = "";
for (var j = 30208; j < 30464; j++)
  i118 += String.fromCharCode(j);
var o118 = i118;
if (i118.replace(/\w+/g, "") !== o118) {
  $ERROR("#118: Error matching character class \w between character 7600 and 76ff");
}

var i119 = "";
for (var j = 30464; j < 30720; j++)
  i119 += String.fromCharCode(j);
var o119 = i119;
if (i119.replace(/\w+/g, "") !== o119) {
  $ERROR("#119: Error matching character class \w between character 7700 and 77ff");
}

var i120 = "";
for (var j = 30720; j < 30976; j++)
  i120 += String.fromCharCode(j);
var o120 = i120;
if (i120.replace(/\w+/g, "") !== o120) {
  $ERROR("#120: Error matching character class \w between character 7800 and 78ff");
}

var i121 = "";
for (var j = 30976; j < 31232; j++)
  i121 += String.fromCharCode(j);
var o121 = i121;
if (i121.replace(/\w+/g, "") !== o121) {
  $ERROR("#121: Error matching character class \w between character 7900 and 79ff");
}

var i122 = "";
for (var j = 31232; j < 31488; j++)
  i122 += String.fromCharCode(j);
var o122 = i122;
if (i122.replace(/\w+/g, "") !== o122) {
  $ERROR("#122: Error matching character class \w between character 7a00 and 7aff");
}

var i123 = "";
for (var j = 31488; j < 31744; j++)
  i123 += String.fromCharCode(j);
var o123 = i123;
if (i123.replace(/\w+/g, "") !== o123) {
  $ERROR("#123: Error matching character class \w between character 7b00 and 7bff");
}

var i124 = "";
for (var j = 31744; j < 32000; j++)
  i124 += String.fromCharCode(j);
var o124 = i124;
if (i124.replace(/\w+/g, "") !== o124) {
  $ERROR("#124: Error matching character class \w between character 7c00 and 7cff");
}

var i125 = "";
for (var j = 32000; j < 32256; j++)
  i125 += String.fromCharCode(j);
var o125 = i125;
if (i125.replace(/\w+/g, "") !== o125) {
  $ERROR("#125: Error matching character class \w between character 7d00 and 7dff");
}

var i126 = "";
for (var j = 32256; j < 32512; j++)
  i126 += String.fromCharCode(j);
var o126 = i126;
if (i126.replace(/\w+/g, "") !== o126) {
  $ERROR("#126: Error matching character class \w between character 7e00 and 7eff");
}

var i127 = "";
for (var j = 32512; j < 32768; j++)
  i127 += String.fromCharCode(j);
var o127 = i127;
if (i127.replace(/\w+/g, "") !== o127) {
  $ERROR("#127: Error matching character class \w between character 7f00 and 7fff");
}

var i128 = "";
for (var j = 32768; j < 33024; j++)
  i128 += String.fromCharCode(j);
var o128 = i128;
if (i128.replace(/\w+/g, "") !== o128) {
  $ERROR("#128: Error matching character class \w between character 8000 and 80ff");
}

var i129 = "";
for (var j = 33024; j < 33280; j++)
  i129 += String.fromCharCode(j);
var o129 = i129;
if (i129.replace(/\w+/g, "") !== o129) {
  $ERROR("#129: Error matching character class \w between character 8100 and 81ff");
}

var i130 = "";
for (var j = 33280; j < 33536; j++)
  i130 += String.fromCharCode(j);
var o130 = i130;
if (i130.replace(/\w+/g, "") !== o130) {
  $ERROR("#130: Error matching character class \w between character 8200 and 82ff");
}

var i131 = "";
for (var j = 33536; j < 33792; j++)
  i131 += String.fromCharCode(j);
var o131 = i131;
if (i131.replace(/\w+/g, "") !== o131) {
  $ERROR("#131: Error matching character class \w between character 8300 and 83ff");
}

var i132 = "";
for (var j = 33792; j < 34048; j++)
  i132 += String.fromCharCode(j);
var o132 = i132;
if (i132.replace(/\w+/g, "") !== o132) {
  $ERROR("#132: Error matching character class \w between character 8400 and 84ff");
}

var i133 = "";
for (var j = 34048; j < 34304; j++)
  i133 += String.fromCharCode(j);
var o133 = i133;
if (i133.replace(/\w+/g, "") !== o133) {
  $ERROR("#133: Error matching character class \w between character 8500 and 85ff");
}

var i134 = "";
for (var j = 34304; j < 34560; j++)
  i134 += String.fromCharCode(j);
var o134 = i134;
if (i134.replace(/\w+/g, "") !== o134) {
  $ERROR("#134: Error matching character class \w between character 8600 and 86ff");
}

var i135 = "";
for (var j = 34560; j < 34816; j++)
  i135 += String.fromCharCode(j);
var o135 = i135;
if (i135.replace(/\w+/g, "") !== o135) {
  $ERROR("#135: Error matching character class \w between character 8700 and 87ff");
}

var i136 = "";
for (var j = 34816; j < 35072; j++)
  i136 += String.fromCharCode(j);
var o136 = i136;
if (i136.replace(/\w+/g, "") !== o136) {
  $ERROR("#136: Error matching character class \w between character 8800 and 88ff");
}

var i137 = "";
for (var j = 35072; j < 35328; j++)
  i137 += String.fromCharCode(j);
var o137 = i137;
if (i137.replace(/\w+/g, "") !== o137) {
  $ERROR("#137: Error matching character class \w between character 8900 and 89ff");
}

var i138 = "";
for (var j = 35328; j < 35584; j++)
  i138 += String.fromCharCode(j);
var o138 = i138;
if (i138.replace(/\w+/g, "") !== o138) {
  $ERROR("#138: Error matching character class \w between character 8a00 and 8aff");
}

var i139 = "";
for (var j = 35584; j < 35840; j++)
  i139 += String.fromCharCode(j);
var o139 = i139;
if (i139.replace(/\w+/g, "") !== o139) {
  $ERROR("#139: Error matching character class \w between character 8b00 and 8bff");
}

var i140 = "";
for (var j = 35840; j < 36096; j++)
  i140 += String.fromCharCode(j);
var o140 = i140;
if (i140.replace(/\w+/g, "") !== o140) {
  $ERROR("#140: Error matching character class \w between character 8c00 and 8cff");
}

var i141 = "";
for (var j = 36096; j < 36352; j++)
  i141 += String.fromCharCode(j);
var o141 = i141;
if (i141.replace(/\w+/g, "") !== o141) {
  $ERROR("#141: Error matching character class \w between character 8d00 and 8dff");
}

var i142 = "";
for (var j = 36352; j < 36608; j++)
  i142 += String.fromCharCode(j);
var o142 = i142;
if (i142.replace(/\w+/g, "") !== o142) {
  $ERROR("#142: Error matching character class \w between character 8e00 and 8eff");
}

var i143 = "";
for (var j = 36608; j < 36864; j++)
  i143 += String.fromCharCode(j);
var o143 = i143;
if (i143.replace(/\w+/g, "") !== o143) {
  $ERROR("#143: Error matching character class \w between character 8f00 and 8fff");
}

var i144 = "";
for (var j = 36864; j < 37120; j++)
  i144 += String.fromCharCode(j);
var o144 = i144;
if (i144.replace(/\w+/g, "") !== o144) {
  $ERROR("#144: Error matching character class \w between character 9000 and 90ff");
}

var i145 = "";
for (var j = 37120; j < 37376; j++)
  i145 += String.fromCharCode(j);
var o145 = i145;
if (i145.replace(/\w+/g, "") !== o145) {
  $ERROR("#145: Error matching character class \w between character 9100 and 91ff");
}

var i146 = "";
for (var j = 37376; j < 37632; j++)
  i146 += String.fromCharCode(j);
var o146 = i146;
if (i146.replace(/\w+/g, "") !== o146) {
  $ERROR("#146: Error matching character class \w between character 9200 and 92ff");
}

var i147 = "";
for (var j = 37632; j < 37888; j++)
  i147 += String.fromCharCode(j);
var o147 = i147;
if (i147.replace(/\w+/g, "") !== o147) {
  $ERROR("#147: Error matching character class \w between character 9300 and 93ff");
}

var i148 = "";
for (var j = 37888; j < 38144; j++)
  i148 += String.fromCharCode(j);
var o148 = i148;
if (i148.replace(/\w+/g, "") !== o148) {
  $ERROR("#148: Error matching character class \w between character 9400 and 94ff");
}

var i149 = "";
for (var j = 38144; j < 38400; j++)
  i149 += String.fromCharCode(j);
var o149 = i149;
if (i149.replace(/\w+/g, "") !== o149) {
  $ERROR("#149: Error matching character class \w between character 9500 and 95ff");
}

var i150 = "";
for (var j = 38400; j < 38656; j++)
  i150 += String.fromCharCode(j);
var o150 = i150;
if (i150.replace(/\w+/g, "") !== o150) {
  $ERROR("#150: Error matching character class \w between character 9600 and 96ff");
}

var i151 = "";
for (var j = 38656; j < 38912; j++)
  i151 += String.fromCharCode(j);
var o151 = i151;
if (i151.replace(/\w+/g, "") !== o151) {
  $ERROR("#151: Error matching character class \w between character 9700 and 97ff");
}

var i152 = "";
for (var j = 38912; j < 39168; j++)
  i152 += String.fromCharCode(j);
var o152 = i152;
if (i152.replace(/\w+/g, "") !== o152) {
  $ERROR("#152: Error matching character class \w between character 9800 and 98ff");
}

var i153 = "";
for (var j = 39168; j < 39424; j++)
  i153 += String.fromCharCode(j);
var o153 = i153;
if (i153.replace(/\w+/g, "") !== o153) {
  $ERROR("#153: Error matching character class \w between character 9900 and 99ff");
}

var i154 = "";
for (var j = 39424; j < 39680; j++)
  i154 += String.fromCharCode(j);
var o154 = i154;
if (i154.replace(/\w+/g, "") !== o154) {
  $ERROR("#154: Error matching character class \w between character 9a00 and 9aff");
}

var i155 = "";
for (var j = 39680; j < 39936; j++)
  i155 += String.fromCharCode(j);
var o155 = i155;
if (i155.replace(/\w+/g, "") !== o155) {
  $ERROR("#155: Error matching character class \w between character 9b00 and 9bff");
}

var i156 = "";
for (var j = 39936; j < 40192; j++)
  i156 += String.fromCharCode(j);
var o156 = i156;
if (i156.replace(/\w+/g, "") !== o156) {
  $ERROR("#156: Error matching character class \w between character 9c00 and 9cff");
}

var i157 = "";
for (var j = 40192; j < 40448; j++)
  i157 += String.fromCharCode(j);
var o157 = i157;
if (i157.replace(/\w+/g, "") !== o157) {
  $ERROR("#157: Error matching character class \w between character 9d00 and 9dff");
}

var i158 = "";
for (var j = 40448; j < 40704; j++)
  i158 += String.fromCharCode(j);
var o158 = i158;
if (i158.replace(/\w+/g, "") !== o158) {
  $ERROR("#158: Error matching character class \w between character 9e00 and 9eff");
}

var i159 = "";
for (var j = 40704; j < 40960; j++)
  i159 += String.fromCharCode(j);
var o159 = i159;
if (i159.replace(/\w+/g, "") !== o159) {
  $ERROR("#159: Error matching character class \w between character 9f00 and 9fff");
}

var i160 = "";
for (var j = 40960; j < 41216; j++)
  i160 += String.fromCharCode(j);
var o160 = i160;
if (i160.replace(/\w+/g, "") !== o160) {
  $ERROR("#160: Error matching character class \w between character a000 and a0ff");
}

var i161 = "";
for (var j = 41216; j < 41472; j++)
  i161 += String.fromCharCode(j);
var o161 = i161;
if (i161.replace(/\w+/g, "") !== o161) {
  $ERROR("#161: Error matching character class \w between character a100 and a1ff");
}

var i162 = "";
for (var j = 41472; j < 41728; j++)
  i162 += String.fromCharCode(j);
var o162 = i162;
if (i162.replace(/\w+/g, "") !== o162) {
  $ERROR("#162: Error matching character class \w between character a200 and a2ff");
}

var i163 = "";
for (var j = 41728; j < 41984; j++)
  i163 += String.fromCharCode(j);
var o163 = i163;
if (i163.replace(/\w+/g, "") !== o163) {
  $ERROR("#163: Error matching character class \w between character a300 and a3ff");
}

var i164 = "";
for (var j = 41984; j < 42240; j++)
  i164 += String.fromCharCode(j);
var o164 = i164;
if (i164.replace(/\w+/g, "") !== o164) {
  $ERROR("#164: Error matching character class \w between character a400 and a4ff");
}

var i165 = "";
for (var j = 42240; j < 42496; j++)
  i165 += String.fromCharCode(j);
var o165 = i165;
if (i165.replace(/\w+/g, "") !== o165) {
  $ERROR("#165: Error matching character class \w between character a500 and a5ff");
}

var i166 = "";
for (var j = 42496; j < 42752; j++)
  i166 += String.fromCharCode(j);
var o166 = i166;
if (i166.replace(/\w+/g, "") !== o166) {
  $ERROR("#166: Error matching character class \w between character a600 and a6ff");
}

var i167 = "";
for (var j = 42752; j < 43008; j++)
  i167 += String.fromCharCode(j);
var o167 = i167;
if (i167.replace(/\w+/g, "") !== o167) {
  $ERROR("#167: Error matching character class \w between character a700 and a7ff");
}

var i168 = "";
for (var j = 43008; j < 43264; j++)
  i168 += String.fromCharCode(j);
var o168 = i168;
if (i168.replace(/\w+/g, "") !== o168) {
  $ERROR("#168: Error matching character class \w between character a800 and a8ff");
}

var i169 = "";
for (var j = 43264; j < 43520; j++)
  i169 += String.fromCharCode(j);
var o169 = i169;
if (i169.replace(/\w+/g, "") !== o169) {
  $ERROR("#169: Error matching character class \w between character a900 and a9ff");
}

var i170 = "";
for (var j = 43520; j < 43776; j++)
  i170 += String.fromCharCode(j);
var o170 = i170;
if (i170.replace(/\w+/g, "") !== o170) {
  $ERROR("#170: Error matching character class \w between character aa00 and aaff");
}

var i171 = "";
for (var j = 43776; j < 44032; j++)
  i171 += String.fromCharCode(j);
var o171 = i171;
if (i171.replace(/\w+/g, "") !== o171) {
  $ERROR("#171: Error matching character class \w between character ab00 and abff");
}

var i172 = "";
for (var j = 44032; j < 44288; j++)
  i172 += String.fromCharCode(j);
var o172 = i172;
if (i172.replace(/\w+/g, "") !== o172) {
  $ERROR("#172: Error matching character class \w between character ac00 and acff");
}

var i173 = "";
for (var j = 44288; j < 44544; j++)
  i173 += String.fromCharCode(j);
var o173 = i173;
if (i173.replace(/\w+/g, "") !== o173) {
  $ERROR("#173: Error matching character class \w between character ad00 and adff");
}

var i174 = "";
for (var j = 44544; j < 44800; j++)
  i174 += String.fromCharCode(j);
var o174 = i174;
if (i174.replace(/\w+/g, "") !== o174) {
  $ERROR("#174: Error matching character class \w between character ae00 and aeff");
}

var i175 = "";
for (var j = 44800; j < 45056; j++)
  i175 += String.fromCharCode(j);
var o175 = i175;
if (i175.replace(/\w+/g, "") !== o175) {
  $ERROR("#175: Error matching character class \w between character af00 and afff");
}

var i176 = "";
for (var j = 45056; j < 45312; j++)
  i176 += String.fromCharCode(j);
var o176 = i176;
if (i176.replace(/\w+/g, "") !== o176) {
  $ERROR("#176: Error matching character class \w between character b000 and b0ff");
}

var i177 = "";
for (var j = 45312; j < 45568; j++)
  i177 += String.fromCharCode(j);
var o177 = i177;
if (i177.replace(/\w+/g, "") !== o177) {
  $ERROR("#177: Error matching character class \w between character b100 and b1ff");
}

var i178 = "";
for (var j = 45568; j < 45824; j++)
  i178 += String.fromCharCode(j);
var o178 = i178;
if (i178.replace(/\w+/g, "") !== o178) {
  $ERROR("#178: Error matching character class \w between character b200 and b2ff");
}

var i179 = "";
for (var j = 45824; j < 46080; j++)
  i179 += String.fromCharCode(j);
var o179 = i179;
if (i179.replace(/\w+/g, "") !== o179) {
  $ERROR("#179: Error matching character class \w between character b300 and b3ff");
}

var i180 = "";
for (var j = 46080; j < 46336; j++)
  i180 += String.fromCharCode(j);
var o180 = i180;
if (i180.replace(/\w+/g, "") !== o180) {
  $ERROR("#180: Error matching character class \w between character b400 and b4ff");
}

var i181 = "";
for (var j = 46336; j < 46592; j++)
  i181 += String.fromCharCode(j);
var o181 = i181;
if (i181.replace(/\w+/g, "") !== o181) {
  $ERROR("#181: Error matching character class \w between character b500 and b5ff");
}

var i182 = "";
for (var j = 46592; j < 46848; j++)
  i182 += String.fromCharCode(j);
var o182 = i182;
if (i182.replace(/\w+/g, "") !== o182) {
  $ERROR("#182: Error matching character class \w between character b600 and b6ff");
}

var i183 = "";
for (var j = 46848; j < 47104; j++)
  i183 += String.fromCharCode(j);
var o183 = i183;
if (i183.replace(/\w+/g, "") !== o183) {
  $ERROR("#183: Error matching character class \w between character b700 and b7ff");
}

var i184 = "";
for (var j = 47104; j < 47360; j++)
  i184 += String.fromCharCode(j);
var o184 = i184;
if (i184.replace(/\w+/g, "") !== o184) {
  $ERROR("#184: Error matching character class \w between character b800 and b8ff");
}

var i185 = "";
for (var j = 47360; j < 47616; j++)
  i185 += String.fromCharCode(j);
var o185 = i185;
if (i185.replace(/\w+/g, "") !== o185) {
  $ERROR("#185: Error matching character class \w between character b900 and b9ff");
}

var i186 = "";
for (var j = 47616; j < 47872; j++)
  i186 += String.fromCharCode(j);
var o186 = i186;
if (i186.replace(/\w+/g, "") !== o186) {
  $ERROR("#186: Error matching character class \w between character ba00 and baff");
}

var i187 = "";
for (var j = 47872; j < 48128; j++)
  i187 += String.fromCharCode(j);
var o187 = i187;
if (i187.replace(/\w+/g, "") !== o187) {
  $ERROR("#187: Error matching character class \w between character bb00 and bbff");
}

var i188 = "";
for (var j = 48128; j < 48384; j++)
  i188 += String.fromCharCode(j);
var o188 = i188;
if (i188.replace(/\w+/g, "") !== o188) {
  $ERROR("#188: Error matching character class \w between character bc00 and bcff");
}

var i189 = "";
for (var j = 48384; j < 48640; j++)
  i189 += String.fromCharCode(j);
var o189 = i189;
if (i189.replace(/\w+/g, "") !== o189) {
  $ERROR("#189: Error matching character class \w between character bd00 and bdff");
}

var i190 = "";
for (var j = 48640; j < 48896; j++)
  i190 += String.fromCharCode(j);
var o190 = i190;
if (i190.replace(/\w+/g, "") !== o190) {
  $ERROR("#190: Error matching character class \w between character be00 and beff");
}

var i191 = "";
for (var j = 48896; j < 49152; j++)
  i191 += String.fromCharCode(j);
var o191 = i191;
if (i191.replace(/\w+/g, "") !== o191) {
  $ERROR("#191: Error matching character class \w between character bf00 and bfff");
}

var i192 = "";
for (var j = 49152; j < 49408; j++)
  i192 += String.fromCharCode(j);
var o192 = i192;
if (i192.replace(/\w+/g, "") !== o192) {
  $ERROR("#192: Error matching character class \w between character c000 and c0ff");
}

var i193 = "";
for (var j = 49408; j < 49664; j++)
  i193 += String.fromCharCode(j);
var o193 = i193;
if (i193.replace(/\w+/g, "") !== o193) {
  $ERROR("#193: Error matching character class \w between character c100 and c1ff");
}

var i194 = "";
for (var j = 49664; j < 49920; j++)
  i194 += String.fromCharCode(j);
var o194 = i194;
if (i194.replace(/\w+/g, "") !== o194) {
  $ERROR("#194: Error matching character class \w between character c200 and c2ff");
}

var i195 = "";
for (var j = 49920; j < 50176; j++)
  i195 += String.fromCharCode(j);
var o195 = i195;
if (i195.replace(/\w+/g, "") !== o195) {
  $ERROR("#195: Error matching character class \w between character c300 and c3ff");
}

var i196 = "";
for (var j = 50176; j < 50432; j++)
  i196 += String.fromCharCode(j);
var o196 = i196;
if (i196.replace(/\w+/g, "") !== o196) {
  $ERROR("#196: Error matching character class \w between character c400 and c4ff");
}

var i197 = "";
for (var j = 50432; j < 50688; j++)
  i197 += String.fromCharCode(j);
var o197 = i197;
if (i197.replace(/\w+/g, "") !== o197) {
  $ERROR("#197: Error matching character class \w between character c500 and c5ff");
}

var i198 = "";
for (var j = 50688; j < 50944; j++)
  i198 += String.fromCharCode(j);
var o198 = i198;
if (i198.replace(/\w+/g, "") !== o198) {
  $ERROR("#198: Error matching character class \w between character c600 and c6ff");
}

var i199 = "";
for (var j = 50944; j < 51200; j++)
  i199 += String.fromCharCode(j);
var o199 = i199;
if (i199.replace(/\w+/g, "") !== o199) {
  $ERROR("#199: Error matching character class \w between character c700 and c7ff");
}

var i200 = "";
for (var j = 51200; j < 51456; j++)
  i200 += String.fromCharCode(j);
var o200 = i200;
if (i200.replace(/\w+/g, "") !== o200) {
  $ERROR("#200: Error matching character class \w between character c800 and c8ff");
}

var i201 = "";
for (var j = 51456; j < 51712; j++)
  i201 += String.fromCharCode(j);
var o201 = i201;
if (i201.replace(/\w+/g, "") !== o201) {
  $ERROR("#201: Error matching character class \w between character c900 and c9ff");
}

var i202 = "";
for (var j = 51712; j < 51968; j++)
  i202 += String.fromCharCode(j);
var o202 = i202;
if (i202.replace(/\w+/g, "") !== o202) {
  $ERROR("#202: Error matching character class \w between character ca00 and caff");
}

var i203 = "";
for (var j = 51968; j < 52224; j++)
  i203 += String.fromCharCode(j);
var o203 = i203;
if (i203.replace(/\w+/g, "") !== o203) {
  $ERROR("#203: Error matching character class \w between character cb00 and cbff");
}

var i204 = "";
for (var j = 52224; j < 52480; j++)
  i204 += String.fromCharCode(j);
var o204 = i204;
if (i204.replace(/\w+/g, "") !== o204) {
  $ERROR("#204: Error matching character class \w between character cc00 and ccff");
}

var i205 = "";
for (var j = 52480; j < 52736; j++)
  i205 += String.fromCharCode(j);
var o205 = i205;
if (i205.replace(/\w+/g, "") !== o205) {
  $ERROR("#205: Error matching character class \w between character cd00 and cdff");
}

var i206 = "";
for (var j = 52736; j < 52992; j++)
  i206 += String.fromCharCode(j);
var o206 = i206;
if (i206.replace(/\w+/g, "") !== o206) {
  $ERROR("#206: Error matching character class \w between character ce00 and ceff");
}

var i207 = "";
for (var j = 52992; j < 53248; j++)
  i207 += String.fromCharCode(j);
var o207 = i207;
if (i207.replace(/\w+/g, "") !== o207) {
  $ERROR("#207: Error matching character class \w between character cf00 and cfff");
}

var i208 = "";
for (var j = 53248; j < 53504; j++)
  i208 += String.fromCharCode(j);
var o208 = i208;
if (i208.replace(/\w+/g, "") !== o208) {
  $ERROR("#208: Error matching character class \w between character d000 and d0ff");
}

var i209 = "";
for (var j = 53504; j < 53760; j++)
  i209 += String.fromCharCode(j);
var o209 = i209;
if (i209.replace(/\w+/g, "") !== o209) {
  $ERROR("#209: Error matching character class \w between character d100 and d1ff");
}

var i210 = "";
for (var j = 53760; j < 54016; j++)
  i210 += String.fromCharCode(j);
var o210 = i210;
if (i210.replace(/\w+/g, "") !== o210) {
  $ERROR("#210: Error matching character class \w between character d200 and d2ff");
}

var i211 = "";
for (var j = 54016; j < 54272; j++)
  i211 += String.fromCharCode(j);
var o211 = i211;
if (i211.replace(/\w+/g, "") !== o211) {
  $ERROR("#211: Error matching character class \w between character d300 and d3ff");
}

var i212 = "";
for (var j = 54272; j < 54528; j++)
  i212 += String.fromCharCode(j);
var o212 = i212;
if (i212.replace(/\w+/g, "") !== o212) {
  $ERROR("#212: Error matching character class \w between character d400 and d4ff");
}

var i213 = "";
for (var j = 54528; j < 54784; j++)
  i213 += String.fromCharCode(j);
var o213 = i213;
if (i213.replace(/\w+/g, "") !== o213) {
  $ERROR("#213: Error matching character class \w between character d500 and d5ff");
}

var i214 = "";
for (var j = 54784; j < 55040; j++)
  i214 += String.fromCharCode(j);
var o214 = i214;
if (i214.replace(/\w+/g, "") !== o214) {
  $ERROR("#214: Error matching character class \w between character d600 and d6ff");
}

var i215 = "";
for (var j = 55040; j < 55296; j++)
  i215 += String.fromCharCode(j);
var o215 = i215;
if (i215.replace(/\w+/g, "") !== o215) {
  $ERROR("#215: Error matching character class \w between character d700 and d7ff");
}

var i216 = "";
for (var j = 55296; j < 55552; j++)
  i216 += String.fromCharCode(j);
var o216 = i216;
if (i216.replace(/\w+/g, "") !== o216) {
  $ERROR("#216: Error matching character class \w between character d800 and d8ff");
}

var i217 = "";
for (var j = 55552; j < 55808; j++)
  i217 += String.fromCharCode(j);
var o217 = i217;
if (i217.replace(/\w+/g, "") !== o217) {
  $ERROR("#217: Error matching character class \w between character d900 and d9ff");
}

var i218 = "";
for (var j = 55808; j < 56064; j++)
  i218 += String.fromCharCode(j);
var o218 = i218;
if (i218.replace(/\w+/g, "") !== o218) {
  $ERROR("#218: Error matching character class \w between character da00 and daff");
}

var i219 = "";
for (var j = 56064; j < 56320; j++)
  i219 += String.fromCharCode(j);
var o219 = i219;
if (i219.replace(/\w+/g, "") !== o219) {
  $ERROR("#219: Error matching character class \w between character db00 and dbff");
}

var i220 = "";
for (var j = 56320; j < 56576; j++)
  i220 += String.fromCharCode(j);
var o220 = i220;
if (i220.replace(/\w+/g, "") !== o220) {
  $ERROR("#220: Error matching character class \w between character dc00 and dcff");
}

var i221 = "";
for (var j = 56576; j < 56832; j++)
  i221 += String.fromCharCode(j);
var o221 = i221;
if (i221.replace(/\w+/g, "") !== o221) {
  $ERROR("#221: Error matching character class \w between character dd00 and ddff");
}

var i222 = "";
for (var j = 56832; j < 57088; j++)
  i222 += String.fromCharCode(j);
var o222 = i222;
if (i222.replace(/\w+/g, "") !== o222) {
  $ERROR("#222: Error matching character class \w between character de00 and deff");
}

var i223 = "";
for (var j = 57088; j < 57344; j++)
  i223 += String.fromCharCode(j);
var o223 = i223;
if (i223.replace(/\w+/g, "") !== o223) {
  $ERROR("#223: Error matching character class \w between character df00 and dfff");
}

var i224 = "";
for (var j = 57344; j < 57600; j++)
  i224 += String.fromCharCode(j);
var o224 = i224;
if (i224.replace(/\w+/g, "") !== o224) {
  $ERROR("#224: Error matching character class \w between character e000 and e0ff");
}

var i225 = "";
for (var j = 57600; j < 57856; j++)
  i225 += String.fromCharCode(j);
var o225 = i225;
if (i225.replace(/\w+/g, "") !== o225) {
  $ERROR("#225: Error matching character class \w between character e100 and e1ff");
}

var i226 = "";
for (var j = 57856; j < 58112; j++)
  i226 += String.fromCharCode(j);
var o226 = i226;
if (i226.replace(/\w+/g, "") !== o226) {
  $ERROR("#226: Error matching character class \w between character e200 and e2ff");
}

var i227 = "";
for (var j = 58112; j < 58368; j++)
  i227 += String.fromCharCode(j);
var o227 = i227;
if (i227.replace(/\w+/g, "") !== o227) {
  $ERROR("#227: Error matching character class \w between character e300 and e3ff");
}

var i228 = "";
for (var j = 58368; j < 58624; j++)
  i228 += String.fromCharCode(j);
var o228 = i228;
if (i228.replace(/\w+/g, "") !== o228) {
  $ERROR("#228: Error matching character class \w between character e400 and e4ff");
}

var i229 = "";
for (var j = 58624; j < 58880; j++)
  i229 += String.fromCharCode(j);
var o229 = i229;
if (i229.replace(/\w+/g, "") !== o229) {
  $ERROR("#229: Error matching character class \w between character e500 and e5ff");
}

var i230 = "";
for (var j = 58880; j < 59136; j++)
  i230 += String.fromCharCode(j);
var o230 = i230;
if (i230.replace(/\w+/g, "") !== o230) {
  $ERROR("#230: Error matching character class \w between character e600 and e6ff");
}

var i231 = "";
for (var j = 59136; j < 59392; j++)
  i231 += String.fromCharCode(j);
var o231 = i231;
if (i231.replace(/\w+/g, "") !== o231) {
  $ERROR("#231: Error matching character class \w between character e700 and e7ff");
}

var i232 = "";
for (var j = 59392; j < 59648; j++)
  i232 += String.fromCharCode(j);
var o232 = i232;
if (i232.replace(/\w+/g, "") !== o232) {
  $ERROR("#232: Error matching character class \w between character e800 and e8ff");
}

var i233 = "";
for (var j = 59648; j < 59904; j++)
  i233 += String.fromCharCode(j);
var o233 = i233;
if (i233.replace(/\w+/g, "") !== o233) {
  $ERROR("#233: Error matching character class \w between character e900 and e9ff");
}

var i234 = "";
for (var j = 59904; j < 60160; j++)
  i234 += String.fromCharCode(j);
var o234 = i234;
if (i234.replace(/\w+/g, "") !== o234) {
  $ERROR("#234: Error matching character class \w between character ea00 and eaff");
}

var i235 = "";
for (var j = 60160; j < 60416; j++)
  i235 += String.fromCharCode(j);
var o235 = i235;
if (i235.replace(/\w+/g, "") !== o235) {
  $ERROR("#235: Error matching character class \w between character eb00 and ebff");
}

var i236 = "";
for (var j = 60416; j < 60672; j++)
  i236 += String.fromCharCode(j);
var o236 = i236;
if (i236.replace(/\w+/g, "") !== o236) {
  $ERROR("#236: Error matching character class \w between character ec00 and ecff");
}

var i237 = "";
for (var j = 60672; j < 60928; j++)
  i237 += String.fromCharCode(j);
var o237 = i237;
if (i237.replace(/\w+/g, "") !== o237) {
  $ERROR("#237: Error matching character class \w between character ed00 and edff");
}

var i238 = "";
for (var j = 60928; j < 61184; j++)
  i238 += String.fromCharCode(j);
var o238 = i238;
if (i238.replace(/\w+/g, "") !== o238) {
  $ERROR("#238: Error matching character class \w between character ee00 and eeff");
}

var i239 = "";
for (var j = 61184; j < 61440; j++)
  i239 += String.fromCharCode(j);
var o239 = i239;
if (i239.replace(/\w+/g, "") !== o239) {
  $ERROR("#239: Error matching character class \w between character ef00 and efff");
}

var i240 = "";
for (var j = 61440; j < 61696; j++)
  i240 += String.fromCharCode(j);
var o240 = i240;
if (i240.replace(/\w+/g, "") !== o240) {
  $ERROR("#240: Error matching character class \w between character f000 and f0ff");
}

var i241 = "";
for (var j = 61696; j < 61952; j++)
  i241 += String.fromCharCode(j);
var o241 = i241;
if (i241.replace(/\w+/g, "") !== o241) {
  $ERROR("#241: Error matching character class \w between character f100 and f1ff");
}

var i242 = "";
for (var j = 61952; j < 62208; j++)
  i242 += String.fromCharCode(j);
var o242 = i242;
if (i242.replace(/\w+/g, "") !== o242) {
  $ERROR("#242: Error matching character class \w between character f200 and f2ff");
}

var i243 = "";
for (var j = 62208; j < 62464; j++)
  i243 += String.fromCharCode(j);
var o243 = i243;
if (i243.replace(/\w+/g, "") !== o243) {
  $ERROR("#243: Error matching character class \w between character f300 and f3ff");
}

var i244 = "";
for (var j = 62464; j < 62720; j++)
  i244 += String.fromCharCode(j);
var o244 = i244;
if (i244.replace(/\w+/g, "") !== o244) {
  $ERROR("#244: Error matching character class \w between character f400 and f4ff");
}

var i245 = "";
for (var j = 62720; j < 62976; j++)
  i245 += String.fromCharCode(j);
var o245 = i245;
if (i245.replace(/\w+/g, "") !== o245) {
  $ERROR("#245: Error matching character class \w between character f500 and f5ff");
}

var i246 = "";
for (var j = 62976; j < 63232; j++)
  i246 += String.fromCharCode(j);
var o246 = i246;
if (i246.replace(/\w+/g, "") !== o246) {
  $ERROR("#246: Error matching character class \w between character f600 and f6ff");
}

var i247 = "";
for (var j = 63232; j < 63488; j++)
  i247 += String.fromCharCode(j);
var o247 = i247;
if (i247.replace(/\w+/g, "") !== o247) {
  $ERROR("#247: Error matching character class \w between character f700 and f7ff");
}

var i248 = "";
for (var j = 63488; j < 63744; j++)
  i248 += String.fromCharCode(j);
var o248 = i248;
if (i248.replace(/\w+/g, "") !== o248) {
  $ERROR("#248: Error matching character class \w between character f800 and f8ff");
}

var i249 = "";
for (var j = 63744; j < 64000; j++)
  i249 += String.fromCharCode(j);
var o249 = i249;
if (i249.replace(/\w+/g, "") !== o249) {
  $ERROR("#249: Error matching character class \w between character f900 and f9ff");
}

var i250 = "";
for (var j = 64000; j < 64256; j++)
  i250 += String.fromCharCode(j);
var o250 = i250;
if (i250.replace(/\w+/g, "") !== o250) {
  $ERROR("#250: Error matching character class \w between character fa00 and faff");
}

var i251 = "";
for (var j = 64256; j < 64512; j++)
  i251 += String.fromCharCode(j);
var o251 = i251;
if (i251.replace(/\w+/g, "") !== o251) {
  $ERROR("#251: Error matching character class \w between character fb00 and fbff");
}

var i252 = "";
for (var j = 64512; j < 64768; j++)
  i252 += String.fromCharCode(j);
var o252 = i252;
if (i252.replace(/\w+/g, "") !== o252) {
  $ERROR("#252: Error matching character class \w between character fc00 and fcff");
}

var i253 = "";
for (var j = 64768; j < 65024; j++)
  i253 += String.fromCharCode(j);
var o253 = i253;
if (i253.replace(/\w+/g, "") !== o253) {
  $ERROR("#253: Error matching character class \w between character fd00 and fdff");
}

var i254 = "";
for (var j = 65024; j < 65280; j++)
  i254 += String.fromCharCode(j);
var o254 = i254;
if (i254.replace(/\w+/g, "") !== o254) {
  $ERROR("#254: Error matching character class \w between character fe00 and feff");
}

var i255 = "";
for (var j = 65280; j < 65536; j++)
  i255 += String.fromCharCode(j);
var o255 = i255;
if (i255.replace(/\w+/g, "") !== o255) {
  $ERROR("#255: Error matching character class \w between character ff00 and ffff");
}

reportCompare(0, 0);
