#ifndef ThirdPartyPaths_h__
#define ThirdPartyPaths_h__

#include <stdint.h>

// These two values are defined in ThirdPartyPaths.cpp, which is a file
// generated by ThirdPartyPaths.py.

extern const char *MOZ_THIRD_PARTY_PATHS[];

extern const uint32_t MOZ_THIRD_PARTY_PATHS_COUNT;

#endif
