// Copyright (C) 2015 André Bargull. All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/*---
es6id: 22.2.4
esid: sec-typedarray-constructors
description: >
  Float32Array is a constructor function.
features: [TypedArray]
---*/

assert.sameValue(typeof Float32Array, 'function', 'typeof Float32Array is "function"');

reportCompare(0, 0);
