// Copyright (c) 2014 the V8 project authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.


/*---
esid: sec-array.prototype.concat
es6id: 22.1.3.1_3
description: Array.prototype.concat no prototype
---*/
assert.sameValue(Array.prototype.concat.prototype, void 0);

reportCompare(0, 0);
